import { __ } from '@wordpress/i18n';

import { Placeholder, TextControl, Button } from "@wordpress/components";
import { grid } from "@wordpress/icons";
import { useEffect } from '@wordpress/element';
import { useSelect, dispatch, useDispatch } from '@wordpress/data';
import {
    useBlockProps,
    useInnerBlocksProps,
    store as blockEditorStore
} from "@wordpress/block-editor";
import {
    createBlocksFromInnerBlocksTemplate
} from "@wordpress/blocks";

import { useState, useRef } from "react";

const ALLOWED_BLOCKS = ["tablestack/table", "tablestack/wootable"];

export default function Edit({attributes, setAttributes, clientId}) {

    const {
        isDefaultTable,
        isProductTable
    } = attributes;

    const blockProps = useBlockProps();

    const innerBlocksProps = useInnerBlocksProps(blockProps, {
        renderAppender: false,
        allowedBlocks: ALLOWED_BLOCKS,
    });

    const { replaceInnerBlocks } = useDispatch(blockEditorStore);

    const [rowCount, setRowCount] = useState(2);
    const [columnCount, setColumnCount] = useState(2);

    const innerBlocks = useSelect(
        (select) => {
            const { getBlocks } = select('core/block-editor');
            return getBlocks(clientId);
        },
        [clientId]
    );
    
    const hasTableBlock = useRef(false);
    
    useEffect(() => {
        if (!hasTableBlock.current && innerBlocks.length > 0) {
            const lastBlock = innerBlocks[innerBlocks.length - 1];
            if (lastBlock && ['tablestack/table', 'tablestack/wootable'].includes(lastBlock.name)) {
                hasTableBlock.current = true;
                dispatch('core/block-editor').selectBlock(lastBlock.clientId);
            }
        }
    }, [innerBlocks]);
    

    function checkWooCommerceIsActive() {
        return typeof window.wc !== 'undefined';
    }

    function handleCreateTable(event, type) {
        event.preventDefault();

        if (rowCount === "" || columnCount === "") return;

        if (type === 'default') {

            const innerBlocks = [
                ["tablestack/table",
                    {rows: rowCount, columns: columnCount},
                    Array.from({ length: rowCount }, (_, rowIndex) => [
                        "tablestack/tablerow",
                            {},
                            Array.from({ length: columnCount }, () => [
                                "tablestack/tablecell",
                            ]),
                    ])]
                ];

            replaceInnerBlocks(
                clientId,
                createBlocksFromInnerBlocksTemplate(innerBlocks)
            );
            setAttributes({
                isDefaultTable: true,
                isProductTable: false,
                rows: rowCount,
                columns: columnCount,
            });
        } else if (type === 'product') {

            if(!checkWooCommerceIsActive()) {
                alert('WooCommerce plugin not installed');
                return
            }

            const innerBlocks = [
                [ 'tablestack/wootable', {} ],
            ];

            replaceInnerBlocks(
                clientId,
                createBlocksFromInnerBlocksTemplate(innerBlocks)
            );
            setAttributes({
                isDefaultTable: false,
                isProductTable: true,
            });
        }
    }

    function updateColumnCount(count) {
        const value = count === "" ? "" : parseInt(count, 10) || 2;
        setColumnCount(value);
    }

    function updateRowCount(count) {
        const value = count === "" ? "" : parseInt(count, 10) || 2;
        setRowCount(value);
    }

    const placeholder = (
        <Placeholder
            label={__("Create Table", "table-block")}
            icon={grid}
            instructions={__("Create a table using block elements", "table-block")}
        >
            <form className="blocks-table__placeholder-form">
                <TextControl
                    __nextHasNoMarginBottom
                    type="number"
                    label={__("Column count", "table-block")}
                    value={columnCount}
                    onChange={updateColumnCount}
                    min="1"
                    className="blocks-table__placeholder-input"
                />
                <TextControl
                    __nextHasNoMarginBottom
                    type="number"
                    label={__("Row count", "table-block")}
                    value={rowCount}
                    onChange={updateRowCount}
                    min="1"
                    className="blocks-table__placeholder-input"
                />
                <Button
                    className="blocks-table__placeholder-button"
                    variant="primary"
                    type="submit"
                    onClick={(e) => handleCreateTable(e, 'default')}
                >
                    {__("Create Table", "table-block")}
                </Button>
                { checkWooCommerceIsActive() &&
                <>
                    <span>{__("Or", "table-block")}</span>
                    <Button
                        className="blocks-table__placeholder-button"
                        variant="primary"
                        type="submit"
                        onClick={(e) => handleCreateTable(e, 'product')}
                    >
                        {__("Product Table", "table-block")}
                    </Button>
                </>
                }
            </form>
        </Placeholder>
    );

    const defaultTable = (
        <>
            <div {...innerBlocksProps} />
        </>
    );

    const wooTable = (
        <>
            <div {...innerBlocksProps} />
        </>
    );

    return (
        <>
            {isDefaultTable ? defaultTable : isProductTable ? wooTable : placeholder}
        </>
    );

}