import {
    createBlocksFromInnerBlocksTemplate,
    createBlock
} from "@wordpress/blocks";
import { __ } from '@wordpress/i18n';
import {
    BlockControls,
    store as blockEditorStore,
    useBlockProps, 
    useInnerBlocksProps,
    InnerBlocks
} from "@wordpress/block-editor";
import { useDispatch, useSelect, select } from "@wordpress/data";
import { ToolbarDropdownMenu } from "@wordpress/components";
import {
    tableRowBefore,
    tableRowDelete,
    tableColumnAfter,
    tableColumnBefore,
    tableColumnDelete,
    table
} from "@wordpress/icons";
import { useEffect } from "@wordpress/element";


const PERMITTED_BLOCKS = window.tableStackConfig.permittedBlocks;

export default function Edit({ attributes, setAttributes, clientId }) {

    const { headerLabel, immediateParentClientId } = attributes;

    useEffect(()=>{
        // Get all parent IDs for the block
        const parentClientIds = select('core/block-editor').getBlockParents(clientId);

        // Get the immediate parent (last in the list of parents)
        const immediateParentId = parentClientIds.length > 0 ? parentClientIds[parentClientIds.length - 1] : null;

        setAttributes({immediateParentClientId:immediateParentId});
    },[])

    

    const innerBlocks = useInnerBlocksProps(
        { className: `cell-container` },
        {
            allowedBlocks: PERMITTED_BLOCKS,
            template: [["core/paragraph"]],
            renderAppender: InnerBlocks.DefaultBlockAppender
        }
    );

    const { insertBlock, updateBlockAttributes, removeBlock } = useDispatch(blockEditorStore);

    const [
        insertRow,
        deleteRow,
        insertColumn,
        deleteColumn,
        isHeader,
        headerValue
    ] = useSelect(
        (select) => {
            const blockStoreSelector = select(blockEditorStore);

            const parentBlocks = blockStoreSelector.getBlockParents(clientId);

            const tableBlockID = parentBlocks.find((blockID) => blockStoreSelector.getBlockName(blockID) === "tablestack/table");

            const tableBlock = blockStoreSelector.getBlock(tableBlockID);

            const selectedRowID = parentBlocks.find((blockID) => blockStoreSelector.getBlockName(blockID) === "tablestack/tablerow");

            const selectedRow = blockStoreSelector.getBlock(selectedRowID);

            const isHeader = selectedRow?.attributes?.isHeader;

            const { rows, columns } = blockStoreSelector.getBlockAttributes(tableBlockID);

            const rowIndex = blockStoreSelector.getBlockIndex(selectedRowID);

            const columnIndex = blockStoreSelector.getBlockIndex(clientId);

            const firstRow = tableBlock.innerBlocks
                .find((block) => block.name === 'tablestack/tablerow');

            const firstRowIsHeader = firstRow?.attributes?.isHeader ?? false;

            let headerCellLabel = '';

            if( firstRowIsHeader ) {
                const headerCell = firstRow?.innerBlocks[columnIndex];
                headerCellLabel = headerCell.innerBlocks?.[0]?.attributes?.content?.text ?? '';
            }



            const insertRow = (after = false) => {
                const newRowIndex = after ? rowIndex + 1 : rowIndex;
                const newRowBlock = createBlocksFromInnerBlocksTemplate([
                    [
                        "tablestack/tablerow",
                        {},
                        Array.from({ length: columns }, () => ["tablestack/tablecell"]),
                    ],
                ])[0];

                insertBlock(newRowBlock, newRowIndex, tableBlockID, true)
                    .then(() => updateBlockAttributes(tableBlockID, { rows: rows + 1 }))
                    .catch((error) => {
                        console.error("Error inserting row:", error);
                    });
            };

            const deleteRow = () => {
                removeBlock(selectedRowID, true)
                    .then(() => updateBlockAttributes(tableBlockID, { rows: rows - 1 }))
                    .catch((error) => {
                        console.error("Error deleting row:", error);
                    });
            };

            const insertColumn = (after = false) => {
                const columnIndex = blockStoreSelector.getBlockIndex(clientId);
                const newColumnIndex = after ? columnIndex + 1 : columnIndex;

                blockStoreSelector
                    .getBlocks(tableBlockID)
                    .forEach((row, index) => {
                        const rowIsHeader = row?.attributes?.isHeader;

                        insertBlock(
                            createBlock("tablestack/tablecell", {
                                htmlTag:
                                    rowIsHeader &&
                                    (index === 0 ||
                                        index ===
                                            tableBlock.innerBlocks.length - 1)
                                        ? "th"
                                        : "td",
                            }),
                            newColumnIndex,
                            row.clientId,
                            false
                        ).catch((error) => {
                            console.error("Error inserting column:", error);
                        });
                    });

                updateBlockAttributes(tableBlockID, {
                    columns: columns + 1,
                }).catch((error) => {
                    console.error("Error updating table attributes:", error);
                });
            };

            const deleteColumn = () => {
                const columnIndex = blockStoreSelector.getBlockIndex(clientId);

                blockStoreSelector
                    .getBlocks(tableBlockID)
                    .forEach((row) => {
                        const cells = blockStoreSelector.getBlockOrder(row.clientId);
                        removeBlock(cells[columnIndex]).catch((error) => {
                            console.error("Error deleting column:", error);
                        });
                    });

                updateBlockAttributes(tableBlockID, {
                    columns: columns - 1,
                }).catch((error) => {
                    console.error("Error updating table attributes:", error);
                });
            };
            

            return [
                insertRow,
                deleteRow,
                insertColumn,
                deleteColumn,
                isHeader,
                headerCellLabel
            ];
        },
        [clientId]
    );

    useEffect(()=>{
        setAttributes({headerLabel: headerValue})
    },[clientId, headerValue])

    const blockProps = headerLabel
        ? useBlockProps({ 'data-label': headerLabel })
        : useBlockProps();

    const handleInsertRowBefore = () => {
        insertRow();
    };
    const handleInsertRowAfter = () => {
        insertRow(true);
    };
    const handleDeleteRow = () => {
        deleteRow();
    };
    const handleInsertColumnBefore = () => {
        insertColumn();
    };
    const handleInsertColumnAfter = () => {
        insertColumn(true);
    };
    const handleDeleteColumn = () => {
        deleteColumn();
    };

    const tableControls = [
        ...(isHeader ?
            [] : [
                    {
                      icon: tableRowBefore,
                      title: __("Insert row before", "table-block"),
                      onClick: handleInsertRowBefore
                    },
                ]
            ),
        {
            icon: tableRowBefore,
            title: __("Insert row after", "table-block"),
            onClick: handleInsertRowAfter
        },
        {
            icon: tableRowDelete,
            title: __("Delete row", "table-block"),
            onClick: handleDeleteRow,
        },
        {
            icon: tableColumnBefore,
            title: __("Insert column before", "table-block"),
            onClick: handleInsertColumnBefore,
        },
        {
            icon: tableColumnAfter,
            title: __("Insert column after", "table-block"),
            onClick: handleInsertColumnAfter,
        },
        {
            icon: tableColumnDelete,
            title: __("Delete column", "table-block"),
            onClick: handleDeleteColumn,
        },
    ];

    const Tag = attributes.htmlTag ?? "td";

    return (
        <>
            <Tag {...blockProps}>
                <div {...innerBlocks}>

                </div>
            </Tag>
            <BlockControls group="other">
                <ToolbarDropdownMenu
                    hasArrowIndicator
                    icon={table}
                    label={__("Edit table",  "table-block")}
                    controls={tableControls}
                />
            </BlockControls>
        </>
    );
}