import {
    useBlockProps,
    useInnerBlocksProps,
    store as blockEditorStore,
    InspectorControls,
    PanelColorSettings,
    BlockControls,
} from "@wordpress/block-editor";

import { __ } from '@wordpress/i18n';

import {
    PanelBody,
    RadioControl,
    ToggleControl,
    __experimentalBoxControl as BoxControl,
    ToolbarDropdownMenu,
    ToolbarGroup,
    __experimentalBorderBoxControl as BorderBoxControl,
    __experimentalDivider as Divider,
 } from "@wordpress/components";

import { useDispatch, useSelect, dispatch } from "@wordpress/data";

import { useEffect } from "@wordpress/element";

import { createBlocksFromInnerBlocksTemplate } from "@wordpress/blocks";

import {
    alignLeft,
    alignRight,
    alignCenter
} from "@wordpress/icons";

import { alignBottom, alignMiddle, alignTop } from "../icons";

const ALLOWED_BLOCKS = ["tablestack/tablerow", "tablestack/tablecell"];

export default function Edit({attributes, setAttributes, clientId}) {

    const {
        showTableHeader,
        showColumnOnlyBorder,
        columnOnlyBorder,
        columns,
        evenRowBackground,
        oddRowBackground,
        headerRowBackground,
        headerRowTextColor,
        oddRowTextColor,
        evenRowTextColor,
        tdPadding,
        verticalAlignment,
        horizontalAlignment,
        tableBorder,
        cellBorder,
        tableOption
    } = attributes;

    const style = {
        '--tablestack-cell-vertical-alignment': verticalAlignment,
        '--tablestack-cell-horizontal-alignment': horizontalAlignment,
        '--tablestack-even-row-bg-color': evenRowBackground,
        '--tablestack-odd-row-bg-color': oddRowBackground,
        '--tablestack-header-row-bg-color': headerRowBackground,
        '--tablestack-header-row-text-color': headerRowTextColor,
        '--tablestack-odd-row-text-color': oddRowTextColor,
        '--tablestack-even-row-text-color': evenRowTextColor,
        '--tablestack-padding-top': tdPadding?.top || '0px',
        '--tablestack-padding-bottom': tdPadding?.bottom || '0px',
        '--tablestack-padding-left': tdPadding?.left || '0px',
        '--tablestack-padding-right': tdPadding?.right || '0px',
        '--tablestack-table-border-style': tableBorder?.style,
        '--tablestack-table-border-color': tableBorder?.color,
        '--tablestack-table-border-width': tableBorder?.width,
        '--tablestack-table-cell-border-style': cellBorder?.style,
        '--tablestack-table-cell-border-color': cellBorder?.color,
        '--tablestack-table-cell-border-width': cellBorder?.width,
        '--tablestack-column-only-border-style': showColumnOnlyBorder ? columnOnlyBorder?.style : '',
        '--tablestack-column-only-border-color': showColumnOnlyBorder ? columnOnlyBorder?.color : '',
        '--tablestack-column-only-border-width': showColumnOnlyBorder ? columnOnlyBorder?.width : '',
    };

    const blockProps = useBlockProps({style});

    const block = useSelect((select) => select(blockEditorStore).getBlock(clientId), [clientId]);

    const { insertBlocks, removeBlock } = useDispatch(blockEditorStore);

    const verticalAlignmentControls = [
        {
            icon: alignTop,
            title: __('Align top', 'table-block'),
            isActive: verticalAlignment === 'top',
            onClick: () => setAttributes({ verticalAlignment: 'top' }),
        },
        {
            icon: alignMiddle,
            title: __('Align middle', 'table-block'),
            isActive: verticalAlignment === 'middle',
            onClick: () => setAttributes({ verticalAlignment: 'middle' }),
        },
        {
            icon: alignBottom,
            title: __('Align bottom', 'table-block'),
            isActive: verticalAlignment === 'bottom',
            onClick: () => setAttributes({ verticalAlignment: 'bottom' }),
        }
    ];

    const horizontalAlignmentControls = [
        {
            icon: alignLeft,
            title: __('Align left', 'table-block'),
            isActive: horizontalAlignment === 'start',
            onClick: () => setAttributes({ horizontalAlignment: 'start' }),
        },
        {
            icon: alignCenter,
            title: __('Align center', 'table-block'),
            isActive: horizontalAlignment === 'center',
            onClick: () => setAttributes({ horizontalAlignment: 'center' }),
        },
        {
            icon: alignRight,
            title: __('Align right', 'table-block'),
            isActive: horizontalAlignment === 'end',
            onClick: () => setAttributes({ horizontalAlignment: 'end' }),
        },
    ];

    useEffect(() => {
        if (showTableHeader) {
            const tableHeaderTemplate = [
                [
                    "tablestack/tablerow",
                    { isHeader: true, className:"header-row" },
                    new Array(columns)
                        .fill(0)
                        .map(() => ["tablestack/tablecell", { htmlTag: "th" }, []]),
                ],
            ];
            insertBlocks(
                createBlocksFromInnerBlocksTemplate(tableHeaderTemplate),
                0,
                clientId
            );
        } else {
            const firstBlock = block.innerBlocks[0];
            const isHeader = firstBlock?.attributes?.isHeader;

            if (isHeader) {
                removeBlock(firstBlock.clientId);
            }
        }
        dispatch('core/block-editor').selectBlock(clientId);
    }, [showTableHeader]);

    const innerBlocks = useInnerBlocksProps(
        {
            renderAppender: false,
            allowedBlocks: ALLOWED_BLOCKS,
        }
    );

    const getVerticalActiveIcon = () => {
        switch (verticalAlignment) {
            case 'top':
                return alignTop;
            case 'middle':
                return alignMiddle;
            case 'bottom':
                return alignBottom;
            default:
                return alignTop;
        }
    };

    const getHorizontalActiveIcon = () => {
        switch (horizontalAlignment) {
            case 'start':
                return alignLeft;
            case 'center':
                return alignCenter;
            case 'end':
                return alignRight;
            default:
                return alignLeft;
        }
    };

    const colors = wp.data.select('core/block-editor').getSettings().colors;

    const selectTableType = ( type ) => {
        setAttributes({ tableOption: type });
        switch (type) {
            case 'general':
                setAttributes({ 
                    showTableHeader: true,
                    headerRowBackground: '#36304A',
                    headerRowTextColor: '#ffffff',
                    evenRowBackground: '#ffffff', 
                    oddRowBackground: '#f5f5f5', 
                    evenRowTextColor: '#000000',
                    oddRowTextColor: '#000000',
                    tableBorder: { color: '', width: '', style: '' },
                    cellBorder: { color: '', width: '', style: '' } 
                });
                break;
            case 'light':
                setAttributes({ 
                    showTableHeader: true,
                    evenRowBackground: '#EDEDED', 
                    oddRowBackground: '#ffffff', 
                    evenRowTextColor: '#000000',
                    oddRowTextColor: '#000000',
                    tableBorder: { color: '', width: '', style: '' },
                    cellBorder: { color: '', width: '', style: '' } 
                });
                break;
            case 'dark':
                setAttributes({ 
                    showTableHeader: true,
                    evenRowBackground: '#2C3844', 
                    oddRowBackground: '#202832', 
                    evenRowTextColor: '#ffffff',
                    oddRowTextColor: '#ffffff',
                    tableBorder: { color: '', width: '', style: '' },
                    cellBorder: { color: '', width: '', style: '' } 
                });
                break;
            default:
                break;
        }
    }

    return (
        <>
            <table { ...blockProps }>
                <tbody {...innerBlocks} />
            </table>
            <BlockControls group="block">
                <ToolbarGroup>
                    <ToolbarDropdownMenu
                        icon={getVerticalActiveIcon()}
                        label={__("Vertical Alignment", "table-block")}
                        controls={verticalAlignmentControls}
                    />
                </ToolbarGroup>
                <ToolbarGroup>
                    <ToolbarDropdownMenu
                        icon={getHorizontalActiveIcon()}
                        label={__("Horizontal Alignment", "table-block")}
                        controls={horizontalAlignmentControls}
                    />
                </ToolbarGroup>
            </BlockControls>
            <InspectorControls group="settings">
                <PanelBody>
                    <ToggleControl
                        checked={showTableHeader}
                        label={__("Show Table Header", "table-block")}
                        onChange={(value) => setAttributes({ showTableHeader: value })}
                    />
                </PanelBody>
                <PanelBody title={__("Table Types", "table-block")}>
                    <RadioControl
                        selected={ tableOption }
                        options={ [
                            { label: 'General', value: 'general' },
                            { label: 'Striped Light', value: 'light' },
                            { label: 'Striped Dark', value: 'dark' }
                        ] }
                        onChange={ ( value ) => selectTableType( value ) }
                    />
                </PanelBody>
            </InspectorControls>

            <InspectorControls group="styles">
                <PanelColorSettings
                    title={__("Table Color Settings", "table-block")}
                    colorSettings={[
                        {
                            value: headerRowBackground,
                            onChange: (newColor) => setAttributes({ headerRowBackground: newColor }),
                            label: __("Header row color", "table-block"),
                        },
                        {
                            value: headerRowTextColor,
                            onChange: (newColor) => setAttributes({ headerRowTextColor: newColor }),
                            label: __("Header row text color", "table-block"),
                        },
                        {
                            value: oddRowBackground,
                            onChange: (newColor) => setAttributes({ oddRowBackground: newColor }),
                            label: __("Odd row color", "table-block"),
                        },
                        {
                            value: oddRowTextColor,
                            onChange: (newColor) => setAttributes({ oddRowTextColor: newColor }),
                            label: __("Odd row text color", "table-block"),
                        },
                        {
                            value: evenRowBackground,
                            onChange: (newColor) => setAttributes({ evenRowBackground: newColor }),
                            label: __("Even row color", "table-block"),
                        },
                        {
                            value: evenRowTextColor,
                            onChange: (newColor) => setAttributes({ evenRowTextColor: newColor }),
                            label: __("Even row text color", "table-block"),
                        },
                    ]}
                />
                <PanelBody title={__("Table Cell Padding", "table-block")}>
                    <BoxControl
                        label={__("Padding", "table-block")}
                        values={tdPadding}
                        onChange={(value) => setAttributes({ tdPadding: value })}
                    />
                </PanelBody>
                <PanelBody title={__("Border Settings", "table-block")} initialOpen={true}>
                    <ToggleControl
                        checked={showColumnOnlyBorder}
                        label={__("Show column only border", "table-block")}
                        onChange={(value) => setAttributes({ showColumnOnlyBorder: value })}
                    />

                    { showColumnOnlyBorder &&
                        <BorderBoxControl
                            __next40pxDefaultSize
                            colors={ colors }
                            label={ __( "Column only border", "table-block" ) }
                            onChange={ (value) => setAttributes({columnOnlyBorder: value}) }
                            value={ columnOnlyBorder }
                        />
                    }
                    <Divider />
                    <BorderBoxControl
                        __next40pxDefaultSize
                        colors={ colors }
                        label={ __( "Table border", "table-block" ) }
                        onChange={ (value) => setAttributes({tableBorder: value}) }
                        value={ tableBorder }
                    />
                    <Divider />
                    <BorderBoxControl
                        __next40pxDefaultSize
                        colors={ colors }
                        label={ __( "Cell border", "table-block" ) }
                        onChange={ (value) => setAttributes({cellBorder: value}) }
                        value={ cellBorder }
                    />
                </PanelBody>
            </InspectorControls>
        </>
    );
}



