import React, { useEffect, useState, useMemo } from 'react';

import { __ } from '@wordpress/i18n';
import {
	flexRender,
	getCoreRowModel,
	useReactTable,
} from '@tanstack/react-table';

import {
	DndContext,
	KeyboardSensor,
	MouseSensor,
	TouchSensor,
	closestCenter,
	useSensor,
	useSensors,
} from '@dnd-kit/core';
import { restrictToHorizontalAxis } from '@dnd-kit/modifiers';
import {
	arrayMove,
	SortableContext,
	horizontalListSortingStrategy,
} from '@dnd-kit/sortable';

import {
	PanelBody,
	SelectControl,
	Button,
	RangeControl,
	__experimentalBoxControl as BoxControl,
	ToolbarDropdownMenu,
	ToolbarGroup,
	ToggleControl,
	RadioControl,
	TextControl
} from '@wordpress/components';

import {
	InspectorControls,
	useBlockProps,
	PanelColorSettings,
    BlockControls,
} from '@wordpress/block-editor';

import {
    alignLeft,
    alignCenter,
    alignRight,
    menu
} from "@wordpress/icons";

import { alignBottom, alignMiddle, alignTop } from "../icons";


import { useSortable } from '@dnd-kit/sortable';
import { CSS } from '@dnd-kit/utilities';

import apiFetch from '@wordpress/api-fetch';
import { addQueryArgs } from '@wordpress/url';
import { decodeEntities } from '@wordpress/html-entities';

import AutocompleteTokenField from './components/autocomplete-tokenfield';

const DraggableTableHeader = ( { header } ) => {
	const { attributes, isDragging, listeners, setNodeRef, transform } =
		useSortable( {
			id: header.column.id,
		} );

	const headerStyle = {
		opacity: isDragging ? 0.8 : 1,
		position: 'relative',
		transform: CSS.Translate.toString( transform ),
		transition: 'width transform 0.2s ease-in-out',
		whiteSpace: 'nowrap',
		width: 'auto',
		zIndex: isDragging ? 1 : 0,
	};

	const convertedText = convertToReadableText(header.column.id)

	return (
		<th colSpan={ header.colSpan } ref={ setNodeRef } style={ headerStyle }>
			<div className="cell-container">
				<div>{ header.isPlaceholder
					? null
					: convertedText }
					<div className='dashicons dashicons-move move-icon' { ...attributes } { ...listeners }></div>
				</div>
			</div>
		</th>
	);
};

const DragAlongCell = ( { cell } ) => {
	const { isDragging, setNodeRef, transform } = useSortable( {
		id: cell.column.id,
	} );

	const dataCellStyle = {
		opacity: isDragging ? 0.8 : 1,
		position: 'relative',
		transform: CSS.Translate.toString( transform ),
		transition: 'width transform 0.2s ease-in-out',
		zIndex: isDragging ? 1 : 0,
	};

	const columnLabel = convertToReadableText(cell.column.id);

	return (
		<td style={ dataCellStyle } ref={ setNodeRef } data-label={columnLabel}>
			<div className="cell-container">
				{ flexRender( cell.column.columnDef.cell, cell.getContext() ) }
			</div>
		</td>
	);
};

const convertToReadableText = (text) => {
	switch(text) {
		case 'image':
			return 'PRODUCT';
		default:
			return text.split('_').map(word => word.charAt(0).toUpperCase() + word.slice(1)).join(' ');
	}
}

const lightenColor = (hex, percent) => {
    // Convert hex to RGB
    let r = parseInt(hex.slice(1, 3), 16);
    let g = parseInt(hex.slice(3, 5), 16);
    let b = parseInt(hex.slice(5, 7), 16);

    // Calculate the lighter color
    r = Math.min(255, Math.floor(r + (255 - r) * percent));
    g = Math.min(255, Math.floor(g + (255 - g) * percent));
    b = Math.min(255, Math.floor(b + (255 - b) * percent));

    // Convert back to hex
    return `#${((1 << 24) + (r << 16) + (g << 8) + b).toString(16).slice(1)}`;
};


const Edit = ( props ) => {

	const { attributes, setAttributes } = props;

	const {
		categories,
		tags,
		numberOfItems,
		order,
		orderBy,
		type,
		productStatus,
		tableColumnOrder,
		tableVisibleColumns,
		evenRowBackground,
        oddRowBackground,
        headerRowBackground,
        tdPadding,
		verticalAlignment,
		horizontalAlignment,
		showSearchInput,
		productImageSize,
		searchInputPosition,
		searchInputWidth,
		noDataMessage,
		cartButtonBgColor,
		cartButtonTextColor,
		cartButtonHoverBgColor,
		cartButtonHoverTextColor,
		paginateButtonBgColor,
		paginateButtonTextColor,
		paginateButtonHoverBgColor,
		paginateButtonHoverTextColor,
		paginateButtonDisabledColor
	} = attributes;


	const [ data, setData ] = useState( [] );
	const [ columns, setColumns ] = useState( [] );
	const [ columnVisibility, setColumnVisibility ] = useState( tableVisibleColumns );
	const [ columnOrder, setColumnOrder ] = useState( tableColumnOrder );
	const [ isLoading, setIsLoading] = useState( false );
	const [ isPaginationLoader, setPaginationLoader] = useState( false );
	const [ lastPage, setLastPage] = useState(1);
    const [ currentPage, setCurrentPage] = useState(1);
	const [ totalItems, setTotalItems] = useState(0);

	const style = {
        '--tablestack-even-row-bg-color': evenRowBackground,
        '--tablestack-odd-row-bg-color': oddRowBackground,
        '--tablestack-header-row-bg-color': headerRowBackground,
        '--tablestack-padding-top': tdPadding?.top || '0px',
        '--tablestack-padding-bottom': tdPadding?.bottom || '0px',
        '--tablestack-padding-left': tdPadding?.left || '0px',
        '--tablestack-padding-right': tdPadding?.right || '0px',
		'--tablestack-cell-horizontal-alignment': horizontalAlignment,
		'--tablestack-cell-vertical-alignment': verticalAlignment,
		'--tablestack-product-image-size': productImageSize,
		'--tablestack-cart-button-bg-color': cartButtonBgColor,
		'--tablestack-cart-button-hover-bg-color': cartButtonHoverBgColor,
		'--tablestack-cart-button-text-color': cartButtonTextColor,
		'--tablestack-cart-button-hover-text-color': cartButtonHoverTextColor,
    };

	const searchStyle = {
		'--tablestack-search-input-width': searchInputWidth,
		'--tablestack-search-input-position': searchInputPosition
	}

	const paginationButtonStyle = {
		'--tablestack-paginate-button-bg-color': paginateButtonBgColor,
		'--tablestack-paginate-button-text-color': paginateButtonTextColor,
		'--tablestack-paginate-button-hover-bg-color': paginateButtonHoverBgColor,
		'--tablestack-paginate-button-hover-text-color': paginateButtonHoverTextColor,
		'--tablestack-paginate-button-disabled-color': paginateButtonDisabledColor
	}

	const hAlignmentControls = [
        {
            icon: alignLeft,
            title: __('Align left', 'table-block'),
            isActive: horizontalAlignment === 'start',
            onClick: () => setAttributes({ horizontalAlignment: 'start' }),
        },
        {
            icon: alignCenter,
            title: __('Align center', 'table-block'),
            isActive: horizontalAlignment === 'center',
            onClick: () => setAttributes({ horizontalAlignment: 'center' }),
        },
        {
            icon: alignRight,
            title: __('Align right', 'table-block'),
            isActive: horizontalAlignment === 'end',
            onClick: () => setAttributes({ horizontalAlignment: 'end' }),
        },
    ];

	const vAlignmentControls = [
        {
            icon: alignTop,
            title: __('Align Top', 'table-block'),
            isActive: verticalAlignment === 'top',
            onClick: () => setAttributes({ verticalAlignment: 'top' }),
        },
        {
            icon: alignMiddle,
            title: __('Align Middle', 'table-block'),
            isActive: verticalAlignment === 'middle',
            onClick: () => setAttributes({ verticalAlignment: 'middle' }),
        },
        {
            icon: alignBottom,
            title: __('Align Bottom', 'table-block'),
            isActive: verticalAlignment === 'bottom',
            onClick: () => setAttributes({ verticalAlignment: 'bottom' }),
        },
    ];

	useEffect( () => {

		const mergedColumns = {
			...tableVisibleColumns,
			...columnVisibility,
		};

		setAttributes( { tableVisibleColumns : mergedColumns } );

	}, [ columnVisibility ] );

	useEffect(() => {
		let paginationDisabledColor = lightenColor(paginateButtonBgColor, .3);
		setAttributes({paginateButtonDisabledColor: paginationDisabledColor});
	},[paginateButtonBgColor])


	useEffect( () => {
		setAttributes({tableColumnOrder: [...columnOrder]})
	}, [ columnOrder ] );


	useEffect(() => {
		if ( data.length > 0 ) {
			const firstRow = data[ 0 ];
			const columnKeys = Object.keys( firstRow );

			const ImageCell = ({ value }) => {
				return (
					<img
						src={value || '/placeholder.jpg'}
						alt="Image"
						onError={(event) => {
							event.target.src = '/placeholder.jpg';
						}}
					/>
				);
			};

			const ShorDescriptionCell = ({ value }) => {

				return (
					<div>
						{value}
					</div>
				);
			};
			const CartButtonCell = () => {

				return (
					<Button className="tablestack-cart-button">
						{__('Add to cart', 'table-block')}
					</Button>
				);
			};

			const dynamicColumns = () => {

				return columnKeys.map((key) => {

					const isImageColumn = key.toLowerCase().includes('image');
					const isShortDescriptionColumn = key.toLowerCase().includes('short_description');
					const isCartButtonColumn = key.toLowerCase().includes('add_to_cart');

					return {
						Header: key,
						accessorKey: key,
						id: key,
						cell: (info) => (
							isImageColumn ? (
								<ImageCell value={info.getValue()} />
							) : isShortDescriptionColumn ? (
								<ShorDescriptionCell value={info.getValue()} />
							) : isCartButtonColumn ? (
								<CartButtonCell value={info.getValue()} />
							) : (
								info.getValue()
							)
						),
					};
				});
			}

			setColumns(dynamicColumns);
		} else {
			const defaultColumns = createColumnsFromVisibility(columnVisibility, columnOrder);
			setColumns(defaultColumns);
		}
	}, [data]);


	const getProducts = ( queryArgs ) => {
		return apiFetch( {
			path: addQueryArgs( `wc/v3/products`, {
				...queryArgs,
			} ),
			parse:false
		} );
	};

	useEffect( () => {
		let params = {
			category: categories.join(','),
			tag: tags.join(','),
			order: order,
			orderby: orderBy,
			...(type && { type }),
			status: productStatus,
			per_page: numberOfItems,
			page: currentPage
		};

		setIsLoading(true);

		getProducts( params )
		.then(response => {
			setLastPage(parseInt(response.headers.get('x-wp-totalpages')))
			setTotalItems(parseInt(response.headers.get('x-wp-total')))
			return response.json()
		})
		.then( ( products ) => {
			const filteredDataArray = products.map( ( product ) => {
				const {
					id,
					name,
					type,
					short_description,
					stock_status,
					average_rating,
					images,
					attributes,
					add_to_cart
				} = product;

				const firstImageUrl = images[ 0 ]?.src;

				const formattedAttributes =
					attributes.length > 0
						? attributes
								.map( ( attribute ) => {
									const attributeName = attribute.name;
									const options = attribute.options.join(', ');
									return `${ attributeName }: ${ options }\n`;
								} )
								.join( '' )
						: '';

				const descriptionWithoutTags = short_description?.replace(/<\/?[^>]+(>|$)/g, '');
				const cartLink = '?add-to-cart=104';

				const stockStatusMap = {
					instock: 'In stock',
					outofstock: 'Out of stock',
					backorder: 'On backorder'
				};

				const formattedProductType = type.charAt(0).toUpperCase() + type.slice(1);

				return {
					id,
					name,
					type: formattedProductType,
					short_description: descriptionWithoutTags,
					stock_status: stockStatusMap[stock_status],
					average_rating,
					image: firstImageUrl,
					attributes: formattedAttributes,
					add_to_cart: cartLink || ''
				};
			} );
			setData( filteredDataArray );
		} ).finally(() => {

			setIsLoading(false);
			setPaginationLoader(false);
		});
	}, [
		order,
		orderBy,
		categories,
		tags,
		type,
		numberOfItems,
		currentPage
	] );

	const createColumnsFromVisibility = (columnVisibility, columnOrder) => {
		const columnHeaders = {
			id: 'ID',
			name: 'Name',
			attributes: 'Attributes',
			image: 'Image',
			short_description: 'Description',
			stock_status: 'Stock Status',
			type: 'Type',
			average_rating: 'Rating',
			add_to_cart: 'Add to Cart'
		};

		const columns = columnOrder
			.filter(columnKey => columnVisibility[columnKey]) // Only include visible columns
			.map(columnKey => ({
				accessorKey: columnKey,
				header: columnHeaders[columnKey] || columnKey
			}));

		return columns;
	};

	const table = useReactTable( {
		data,
		columns,
		getCoreRowModel: getCoreRowModel(),
		state: {
			columnVisibility,
			columnOrder,
		},
		onColumnOrderChange: setColumnOrder,
		onColumnVisibilityChange: setColumnVisibility,
		manualPagination: true,
		debugTable: true,
		debugHeaders: true,
		debugColumns: true,
	} );


	function handleDragEnd( event ) {
		const { active, over } = event;
		if ( active && over && active.id !== over.id ) {
			setColumnOrder( ( columnOrder ) => {
				const oldIndex = columnOrder.indexOf( active.id );
				const newIndex = columnOrder.indexOf( over.id );
				return arrayMove( columnOrder, oldIndex, newIndex );
			} );
		}
	}

	const handleCheckboxChange = (event) => {
		const { name, checked } = event.target;

		setColumnVisibility({
		  	...columnVisibility,
		  	[name]: checked,
		});
	};


	const sensors = useSensors(
		useSensor( MouseSensor, {} ),
		useSensor( TouchSensor, {} ),
		useSensor( KeyboardSensor, {} )
	);

	const getCategoryTitle = category =>
        decodeEntities(category.name) || __('(no title)', 'table-block');

    const fetchCategorySuggestions = search => {
        return apiFetch({
            path: addQueryArgs('wc/v3/products/categories', {
                search,
                per_page: 20,
                _fields: 'id,name,parent',
                orderby: 'count',
                order: 'desc',
            }),
        }).then(categories =>
            Promise.all(
                categories.map(category => {
                    if (category.parent > 0) {
                        return apiFetch({
                            path: addQueryArgs(`wc/v3/products/categories/${category.parent}`, {
                                _fields: 'name',
                            }),
                        }).then(parentCategory => ({
                            value: category.id,
                            label: `${getCategoryTitle(category)} – ${getCategoryTitle(parentCategory)}`,
                        }));
                    }
                    return Promise.resolve({
                        value: category.id,
                        label: getCategoryTitle(category),
                    });
                })
            )
        );
    };

    const fetchSavedCategories = categoryIDs => {
        return apiFetch({
            path: addQueryArgs('wc/v3/products/categories', {
                per_page: 100,
                _fields: 'id,name',
                include: categoryIDs.join(','),
            }),
        }).then(function (categories) {
            return categories.map(category => ({
                value: category.id,
                label: decodeEntities(category.name) || __('(no title)', 'table-block'),
            }));
        });
    };

	const fetchTagSuggestions = search => {
        return apiFetch({
            path: addQueryArgs('wc/v3/products/tags', {
                search,
                per_page: 20,
                _fields: 'id,name',
                orderby: 'count',
                order: 'desc',
            }),
        }).then(function (tags) {
            return tags.map(tag => ({
                value: tag.id,
                label: decodeEntities(tag.name) || __('(no title)', 'table-block'),
            }));
        });
    };

    const fetchSavedTags = tagIDs => {
        return apiFetch({
            path: addQueryArgs('wc/v3/products/tags', {
                per_page: 100,
                _fields: 'id,name',
                include: tagIDs.join(','),
            }),
        }).then(function (tags) {
            return tags.map(tag => ({
                value: tag.id,
                label: decodeEntities(tag.name) || __('(no title)', 'table-block'),
            }));
        });
    };

	const blockProps = useBlockProps({style});
	const searchProps = useBlockProps({
		style: searchStyle,
		className: "tablestack-product-table-search"
	});
	const paginationProps = useBlockProps({
		style: paginationButtonStyle,
		className: "tablestack-pagination"
	})

	return (
		<>
			<BlockControls group="block">
				<ToolbarGroup>
                    <ToolbarDropdownMenu
                        icon={alignMiddle}
                        label={__("Vertical Alignment", "table-block")}
                        controls={vAlignmentControls}
                    />
                </ToolbarGroup>
                <ToolbarGroup>
                    <ToolbarDropdownMenu
                        icon={menu}
                        label={__("Horizontal Alignment", "table-block")}
                        controls={hAlignmentControls}
                    />
                </ToolbarGroup>
            </BlockControls>

			<InspectorControls group="settings">
				<PanelBody title={ __( 'Search settings', 'table-block' ) }>
                    <ToggleControl
                        checked={showSearchInput}
                        label={__("Show search input", "table-block")}
                        onChange={(value) => setAttributes({ showSearchInput: value })}
                    />
					{showSearchInput &&
					<>
						<RangeControl
							label={__("Search Input Width", "table-block") }
							value={parseInt(searchInputWidth)}
							onChange={(value) => setAttributes({ searchInputWidth: `${value}%` })}
							min={10}
							max={100}
						/>
						<RadioControl
							label={ __( 'Search Input Alignment', 'table-block' ) }
							selected={ searchInputPosition }
							options={ [
							{ label: 'Left', value: 'left' },
							{ label: 'Right', value: 'right' },
							] }
							onChange={ ( value ) => setAttributes( {searchInputPosition: value} ) }
						/>
					</>
					}
                </PanelBody>
				<PanelBody title={ __( 'Column settings', 'table-block' ) }>
					<div className="px-1 border-b border-black">
						{Object.keys(columnVisibility).map((propertyName) => (
							<div key={propertyName}>
								<input
									type="checkbox"
									id={propertyName}
									name={propertyName}
									checked={columnVisibility[propertyName]}
									onChange={handleCheckboxChange}
								/>
								<label htmlFor={propertyName}>{convertToReadableText(propertyName)}</label>
							</div>
						))}
					</div>
				</PanelBody>
				<PanelBody title={ __( 'Sorting and filtering', 'table-block' ) }>
					<RangeControl
						label={__('Items per page', 'table-block')}
						value={numberOfItems}
						onChange={(value) =>
							setAttributes({ numberOfItems: value })
						}
						min={1}
					/>


					<AutocompleteTokenField
						key="Categories"
						tokens={categories || []}
						onChange={(categories) => setAttributes({ categories })}
						fetchSuggestions={fetchCategorySuggestions}
						fetchSavedInfo={fetchSavedCategories}
						label={ __( 'Categories', 'table-block') }
						help={ __('Begin typing category title, click autocomplete result to select.', 'table-block') }
					/>

					<AutocompleteTokenField
						key="tags"
						tokens={tags || []}
						onChange={(tags) => setAttributes({ tags })}
						fetchSuggestions={fetchTagSuggestions}
						fetchSavedInfo={fetchSavedTags}
						label={ __( 'Tags', 'table-block') }
						help={ __('Begin typing tags name, click autocomplete result to select.', 'table-block') }
					/>

					<SelectControl
						label={ __( 'Order', 'table-block' ) }
						value={order}
						options={[
							{ label: __( 'Ascending', 'table-block' ), value: 'asc' },
							{ label: __( 'Descending', 'table-block' ), value: 'desc' }
						]}

						onChange={ ( order ) => setAttributes( { order } ) }
						__nextHasNoMarginBottom
					/>

					<SelectControl
						label={ __('Order by', 'table-block')}
						value={orderBy}
						options={[
							{ label: __( 'ID', 'table-block' ), value: 'id' },
							{ label: __( 'Date', 'table-block' ), value: 'date' },
							{ label: __( 'Modified', 'table-block' ), value: 'modified' },
							{ label: __( 'Title', 'table-block' ), value: 'title' },
							{ label: __( 'Slug', 'table-block' ), value: 'slug' },
							{ label: __( 'Price', 'table-block' ), value: 'price' },
							{ label: __( 'Popularity', 'table-block' ), value: 'popularity' },
							{ label: __( 'Rating', 'table-block' ), value: 'rating' },
							{ label: __( 'Menu order', 'table-block' ), value: 'menu_order' },
							{ label: __( 'Comment count', 'table-block' ), value: 'comment_count' },
						]}

						onChange={( orderBy ) => setAttributes( { orderBy } )}
						__nextHasNoMarginBottom
					/>

					<SelectControl
						label={ __('Type', 'table-block')}
						value={type}
						options={[
							{ label: __( 'Simple', 'table-block' ), value: 'simple' },
							{ label: __( 'Grouped', 'table-block' ), value: 'grouped' },
							{ label: __( 'External', 'table-block' ), value: 'external' },
							{ label: __( 'Variable', 'table-block' ), value: 'variable' },
							{ label: __( 'Any', 'table-block' ), value: '' },
						]}

						onChange={( type ) => setAttributes( { type } )}
						__nextHasNoMarginBottom
					/>
					<TextControl
						label={ __('Custom Message if no data', 'table-block')}
						__nextHasNoMarginBottom
						onChange={(value) => setAttributes({ noDataMessage: value })}
						value={noDataMessage}
					/>

				</PanelBody>
			</InspectorControls>
			<InspectorControls group="styles">
				<PanelBody title="Product Image Settings" initialOpen={true}>
                    <RangeControl
                        label="Image Width (%)"
                        value={parseInt(productImageSize)}
                        onChange={(value) => setAttributes({ productImageSize: `${value}%` })}
                        min={10}
                        max={100}
                    />
                </PanelBody>
                <PanelColorSettings
                    title={__("Table Color Settings", "table-block")}
                    colorSettings={[
                        {
                            value: headerRowBackground,
                            onChange: (newColor) => setAttributes({ headerRowBackground: newColor }),
                            label: __("Header row color", "table-block"),
                        },
                        {
                            value: oddRowBackground,
                            onChange: (newColor) => setAttributes({ oddRowBackground: newColor }),
                            label: __("Odd row color", "table-block"),
                        },
                        {
                            value: evenRowBackground,
                            onChange: (newColor) => setAttributes({ evenRowBackground: newColor }),
                            label: __("Even row color", "table-block"),
                        }
                    ]}
                />
				<PanelColorSettings
					title={ __( 'Cart button color settings', 'table-block' ) }
					initialOpen={ false }
					colorSettings={ [
						{
							value: cartButtonBgColor,
							onChange: ( cartButtonBgColor ) => setAttributes( { cartButtonBgColor } ),
							label: __( 'Background Color', 'table-block' ),
						},
						{
							value: cartButtonTextColor,
							onChange: ( cartButtonTextColor ) => setAttributes( { cartButtonTextColor } ),
							label: __( 'Text Color', 'table-block' ),
						},
						{
							value: cartButtonHoverBgColor,
							onChange: ( cartButtonHoverBgColor ) => setAttributes( { cartButtonHoverBgColor } ),
							label: __( 'Hover Background Color', 'table-block' ),
						},
						{
							value: cartButtonHoverTextColor,
							onChange: ( cartButtonHoverTextColor ) => setAttributes( { cartButtonHoverTextColor } ),
							label: __( 'Hover Text Color', 'table-block' ),
						},
					] }
				></PanelColorSettings>
				<PanelColorSettings
					title={ __( 'Pagination button color settings', 'table-block' ) }
					initialOpen={ false }
					colorSettings={ [
						{
							value: paginateButtonBgColor,
							onChange: ( paginateButtonBgColor ) => setAttributes( { paginateButtonBgColor } ),
							label: __( 'Background Color', 'table-block' ),
						},
						{
							value: paginateButtonTextColor,
							onChange: ( paginateButtonTextColor ) => setAttributes( { paginateButtonTextColor } ),
							label: __( 'Text Color', 'table-block' ),
						},
						{
							value: paginateButtonHoverBgColor,
							onChange: ( paginateButtonHoverBgColor ) => setAttributes( { paginateButtonHoverBgColor } ),
							label: __( 'Hover Background Color', 'table-block' ),
						},
						{
							value: paginateButtonHoverTextColor,
							onChange: ( paginateButtonHoverTextColor ) => setAttributes( { paginateButtonHoverTextColor } ),
							label: __( 'Hover Text Color', 'table-block' ),
						}
					] }
				></PanelColorSettings>
                <PanelBody title={__("Table Cell Padding", "table-block")}>
                    <BoxControl
                        label={__("Padding", "table-block")}
                        values={tdPadding}
                        onChange={(value) => setAttributes({ tdPadding: value })}
                    />
                </PanelBody>
            </InspectorControls>

			{ showSearchInput &&
                <div {...searchProps}>
                  <input type='text' className="tablestack-search-input" placeholder="Search products..." />
                </div>
            }

				<div className="tablestack-table-container">
					<DndContext
						collisionDetection={ closestCenter }
						modifiers={ [ restrictToHorizontalAxis ] }
						onDragEnd={ handleDragEnd }
						sensors={ sensors }
					>
						<table {...blockProps}>
							<thead>
								{ table
									.getHeaderGroups()
									.map( ( headerGroup ) => (
										<>

										<tr key={ headerGroup.id } className="header-row">

											<SortableContext
												items={ columnOrder }
												strategy={
													horizontalListSortingStrategy
												}
											>
												{ headerGroup.headers.map(
													( header ) => (
														<DraggableTableHeader
															key={ header.id }
															header={ header }
														/>
													)
												) }
											</SortableContext>
										</tr>
										{ isLoading && !isPaginationLoader &&
												<tr>
													<th style={{textAlign: 'center', whiteSpace: 'nowrap'}}>
														<div>{__('Loading...','table-block')}</div>
													</th>
												</tr>
											}
										</>
									) ) }

							</thead>
							{ !isLoading && data.length > 0 &&
							<tbody>
								{ table.getRowModel().rows.map( ( row, index ) => (

									<tr key={ row.id } className={index % 2 === 0 ? 'odd-row' : 'even-row'}>
										{ row
											.getVisibleCells()
											.map( ( cell ) => (
												<SortableContext
													key={ cell.id }
													items={ columnOrder }
													strategy={
														horizontalListSortingStrategy
													}
												>
													<DragAlongCell
														key={ cell.id }
														cell={ cell }
													/>
												</SortableContext>
											) ) }
									</tr>
								) ) }
							</tbody>
							}

							{ !isLoading && data.length < 1  &&
								<tbody>
									<tr>
										<td className='warning'>
											{noDataMessage}
										</td>
									</tr>
								</tbody>
							}
						</table>

					</DndContext>
				</div>

			{ numberOfItems < totalItems &&
				<div {...paginationProps}>
					<button
						type="button"
						className="paginate-button mr-5"
						onClick={() => {
							setCurrentPage(currentPage - 1)
							setPaginationLoader(true)
						}}
						disabled={currentPage === 1}
					>
						{__('Previous','table-block')}
					</button>
					<button
						className="paginate-button"
						onClick={() => {
							setCurrentPage(currentPage + 1)
							setPaginationLoader(true)
						}}
						disabled={currentPage === lastPage}
					>
						{__('Next','table-block')}
					</button>
				</div>
			}
		</>
	);
}

export default Edit;
