  import {
    flexRender,
    getCoreRowModel,
    useReactTable,
    getSortedRowModel,
    getFilteredRowModel,
  } from "@tanstack/react-table";
  import React, { useEffect, useState, useRef } from 'react'
  import ReactDOM from "react-dom/client"
  import apiFetch from '@wordpress/api-fetch'
  import { addQueryArgs } from '@wordpress/url'
  import { __ } from '@wordpress/i18n';


  const convertToReadableText = (text) => {
    switch(text) {
      case 'image':
        return 'PRODUCT';
      default:
        return text.split('_').map(word => word.charAt(0).toUpperCase() + word.slice(1)).join(' ');
    }
  }

    function Table({
      tableColumnsOrder,
      tableVisibleColumns,
      tableSearch,
      productOrder,
      productOrderBy,
      productType,
      productStatus,
      productCategories,
      productTags,
      productPerPage,
      customMessage
     }) {


      const [ data, setData ] = useState( [] );
      const [ columns, setColumns ] = useState( [] );
      const [ columnVisibility, setColumnVisibility ] = useState( tableVisibleColumns );
      const [ columnOrder, setColumnOrder ] = useState( tableColumnsOrder );
      const [ columnFilters, setColumnFilters ] = React.useState([])
      const [ globalFilter, setGlobalFilter ] = React.useState("")
      const [ lastPage, setLastPage] = useState(1);
      const [ currentPage, setCurrentPage] = useState(1);
      const [ isHeaderLoading, setIsHeaderLoading] = useState( true );
      const [ isBodyLoading, setIsBodyLoading] = useState( false );
      const [ totalItems, setTotalItems] = useState(0);

      const stockStatusMap = {
        instock: 'In stock',
        outofstock: 'Out of stock',
        backorder: 'On backorder'
      };

      useEffect(() => {

        if ( data.length > 0 ) {

          const ImageCell = ({ value }) => {

            return (
              <img
              src={value || '/placeholder.jpg'}
              alt="Image"
              onError={(event) => {
                event.target.src = '/placeholder.jpg';
              }}
              />
            );
          };

          const StockStatusCell = ({ value }) => {

            return (
              <div>
                {stockStatusMap[value]}
              </div>
            );
          };

          const ProductTypeCell = ({ value }) => {

            return (
              <div>
                {value.charAt(0).toUpperCase() + value.slice(1)}
              </div>
            );
          };

          const ShorDescriptionCell = ({ value }) => {

            return (
              <div>
                {value}
              </div>
            );
          };
          const CartButtonCell = ({ value }) => {

            const buttonRef = useRef(null);
            const productId = value.id;
            const productType = value.type;


            let buttonText = __('Add to Cart','table-block');
            let buttonHref = `?add-to-cart=${productId}`;


            if (productType === 'variable') {
              buttonText = __('Select Options','table-block');
              buttonHref = value.add_to_cart;
            } else if (productType === 'grouped') {
              buttonText = __('View Products','table-block');
              buttonHref = value.add_to_cart;
            }

            const ajaxClass = productType === 'simple' ? 'ajax_add_to_cart add_to_cart_button' : '';

            useEffect(() => {
              if (!buttonRef.current) return;

              const observer = new MutationObserver((mutationsList) => {
                mutationsList.forEach((mutation) => {
                  if (mutation.attributeName === 'class') {
                    const targetElement = mutation.target;
                    if (targetElement.classList.contains('loading')) {
                      targetElement.innerHTML = __('Adding...','table-block');
                    } else if (targetElement.classList.contains('added')) {
                      targetElement.innerHTML = __('Added to Cart','table-block');
                    }
                  }
                });
              });


              observer.observe(buttonRef.current, { attributes: true, attributeFilter: ['class'] });


              return () => {
                observer.disconnect();
              };
            }, []);


            return (
                <a
                  className={`${ajaxClass} tablestack-cart-button`}
                  href={buttonHref}
                  data-product_id={productId}
                  aria-label={buttonText}
                  rel="nofollow"
                  ref={buttonRef}
                >
                  {buttonText}
                </a>
            )

          };

          const firstRow = data[ 0 ];
          const columnKeys = Object.keys( firstRow );


          const dynamicColumns = () => {

            return columnKeys.map((key) => {
              const isImageColumn = key.toLowerCase().includes('image');
              const isShortDescriptionColumn = key.toLowerCase().includes('short_description');
              const isCartButtonColumn = key.toLowerCase().includes('add_to_cart');
              const isStockColumn = key.toLowerCase().includes('stock_status');
              const isTypeColumn = key.toLowerCase().includes('type');

              return {
                Header: key,
                accessorKey: key,
                id: key,
                enableSorting: isCartButtonColumn ? false : true,
                filterFn: "includesString",
                cell: (info) => (
                  isImageColumn ? (
                    <ImageCell value={info.getValue()} />
                  ) : isShortDescriptionColumn ? (
                    <ShorDescriptionCell value={info.getValue()} />
                  ) : isCartButtonColumn ? (
                    <CartButtonCell value={info.row.original}/>
                  ) : isStockColumn ? (
                    <StockStatusCell value={info.getValue()}/>
                  ) : isTypeColumn ? (
                    <ProductTypeCell value={info.getValue()}/>
                  ) : (
                    info.getValue()
                  )
                ),
              };
            });
          }

          setColumns(dynamicColumns);
        }  else {
          const defaultColumns = createColumnsFromVisibility(columnVisibility, columnOrder);
          setColumns(defaultColumns);
        }
      }, [data]);

      const getProducts = ( queryArgs ) => {
        return apiFetch( {
          path: addQueryArgs( `wc/v3/products`, {
            ...queryArgs
          } ),
          parse:false
        } );
      };

      useEffect( () => {
        let params = {
          category: productCategories.join(','),
          tag: productTags.join(','),
          order: productOrder,
          orderby: productOrderBy,
          per_page: productPerPage,
          status: productStatus,
          search: globalFilter
        };

        if(productType){
          params.type = productType
        }

        if(!globalFilter) {
          params.page = currentPage
        }


        setIsBodyLoading(true);

        getProducts( params )
        .then(response => {
          setLastPage(parseInt(response.headers.get('x-wp-totalpages')))
          setTotalItems(parseInt(response.headers.get('x-wp-total')))
          return response.json()
        })
        .then( ( products ) => {

          const filteredDataArray = products.map( ( product ) => {
            const {
              id,
              name,
              type,
              short_description,
              stock_status,
              average_rating,
              images,
              attributes,
              add_to_cart,
              permalink
            } = product;


            const firstImageUrl = images[ 0 ]?.src;

            const formattedAttributes =
              attributes.length > 0
                ? attributes
                    .map( ( attribute ) => {
                      const attributeName = attribute.name;
                      const options = attribute.options.join(', ');
                      return `${ attributeName }: ${ options }\n`;
                    } )
                    .join( '' )
                : '';

            const descriptionWithoutTags = short_description?.replace(/<\/?[^>]+(>|$)/g, '');
            const productPermalink = permalink;

            return {
              id,
              name,
              type,
              short_description: descriptionWithoutTags,
              stock_status,
              average_rating,
              image: firstImageUrl,
              attributes: formattedAttributes,
              add_to_cart: productPermalink
            };
          } );
          setData( filteredDataArray );
        } ).finally(() => {

          setIsBodyLoading(false);
          setIsHeaderLoading(false);
        });
      }, [ currentPage, globalFilter ] );

      const createColumnsFromVisibility = (columnVisibility, columnOrder) => {
        const columnHeaders = {
          id: 'ID',
          name: 'Name',
          attributes: 'Attributes',
          image: 'Image',
          short_description: 'Description',
          stock_status: 'Stock Status',
          type: 'Type',
          average_rating: 'Rating',
          add_to_cart: 'Add to Cart'
        };

        const columns = columnOrder
          .filter(columnKey => columnVisibility[columnKey]) // Only include visible columns
          .map(columnKey => ({
            accessorKey: columnKey,
            header: columnHeaders[columnKey] || columnKey
          }));

        return columns;
      };

        const table = useReactTable({
            data,
            columns,
            getCoreRowModel: getCoreRowModel(),
            getSortedRowModel: getSortedRowModel(),
            state: {
              columnVisibility,
              columnOrder,
              columnFilters,
              globalFilter
            },
            onGlobalFilterChange: setGlobalFilter,
            globalFilterFn: "fuzzy",
            getFilteredRowModel: getFilteredRowModel(),
            manualPagination: true,
            manualFiltering: true,
            debugTable: true,
            debugHeaders: true,
            debugColumns: false
        });

        const removeAddedToCartElements = () => {
          const elements = document.querySelectorAll('.added_to_cart');
          elements.forEach((element) => {
            element.parentNode.removeChild(element);
          });
        };

        const headerStyle = {
          position: 'relative',
          transition: 'width transform 0.2s ease-in-out',
          whiteSpace: 'nowrap',
          width: 'auto',
        };

        return (
          <>

              { tableSearch && data.length > 0 &&
                <div className="tablestack-product-table-search">
                  <DebouncedInput
                    value={globalFilter ?? ""}
                    onChange={value => {
                      setGlobalFilter(String(value));
                    }}
                    className="tablestack-search-input"
                    placeholder="Search products..."
                  />
                </div>
              }
              <div className="frontend-table-wrapper">
                <table className="wp-block-tablestack-wootable">

                  <thead>
                  {isHeaderLoading ? (
                    <tr>
                      <th colSpan="100%">Generating headers...</th>
                    </tr>
                      ) : (
                        table.getHeaderGroups().map(headerGroup => (
                          <tr key={headerGroup.id} className="header-row">
                            {headerGroup.headers.map(header => {
                              let sortClass = header.column.getCanSort()
                                ? `select-none dashicons ${
                                    header.column.getIsSorted() === 'asc'
                                      ? 'dashicons-arrow-up'
                                      : header.column.getIsSorted() === 'desc'
                                      ? 'dashicons-arrow-down'
                                      : 'dashicons-sort'
                                  }`
                                : '';
                              return (
                                <th
                                  key={header.id}
                                  colSpan={header.colSpan}
                                  style={ headerStyle }
                                >
                                  <div className="cell-container">
                                    {header.isPlaceholder ? null : (
                                      <>
                                        <div
                                          {...{
                                            className: 'header-cell',
                                            onClick: header.column.getToggleSortingHandler(),
                                          }}
                                        >
                                          <span>{convertToReadableText(header.id)}</span>
                                          <div className={sortClass}></div>
                                        </div>
                                      </>
                                    )}
                                  </div>
                                </th>
                              );
                            })}
                          </tr>
                        ))
                      )}
                  </thead>

                  <tbody className={isBodyLoading ? 'body-loading' : ''}>

                    { data.length > 0 ? (
                      table.getRowModel().rows.map((row, index) => (
                        <tr key={row.id} className={index % 2 === 0 ? 'odd-row' : 'even-row'}>
                          {row.getVisibleCells().map(cell => (
                            <td key={cell.id} data-label={convertToReadableText(cell.column.id)}>
                              <div className="cell-container">
                                {flexRender(cell.column.columnDef.cell, cell.getContext())}
                              </div>
                            </td>
                          ))}
                        </tr>
                      ))
                    ) : (
                      <tr className={ !isBodyLoading && data.length < 1 ? 'warning' : '' }>
                        <td colSpan="100%">
                          {isBodyLoading && data.length < 1 ?
                          <b>{__('loading...','table-block')}</b> : customMessage
                          }
                        </td>
                      </tr>
                    )}
                  </tbody>

                </table>
              </div>


            { productPerPage < totalItems  && !isBodyLoading &&
              <div className="tablestack-pagination">
                <button
                  className="paginate-button mr-5"
                  onClick={() => {
                    removeAddedToCartElements()
                    setCurrentPage(currentPage - 1)
                  }}
                  disabled={currentPage === 1}
                >
                  {__('Previous','table-block')}
                </button>
                <button
                  className="paginate-button"
                  onClick={() => {
                    removeAddedToCartElements()
                    setCurrentPage(currentPage + 1)
                  }}
                  disabled={currentPage === lastPage}
                >
                  {__('Next','table-block')}
                </button>
              </div>
            }
          </>
        );
    }


      function DebouncedInput({
        value: initialValue,
        onChange,
        debounce = 700,
        ...props
      }) {
        const [value, setValue] = React.useState(initialValue)

        React.useEffect(() => {
          setValue(initialValue)
        }, [initialValue])

        React.useEffect(() => {
          const timeout = setTimeout(() => {
            onChange(value)
          }, debounce)

          return () => clearTimeout(timeout)
        }, [value])

        return (
          <input {...props} value={value} onChange={e => setValue(e.target.value)} />
        )
      }


    const blockContainers = document.querySelectorAll('.tablestack-wootable');

    blockContainers.forEach((container, index) => {

      if (container.dataset.blockType === 'tablestack-wootable') {
          const visibileColumns = JSON.parse(container.dataset.tableVisibility);
          const columnsOrder = JSON.parse(container.dataset.tableColumns);
          const tableSearch = JSON.parse(container.dataset.tableSearch);
          const productOrder = JSON.parse(container.dataset.blockProductorder);
          const productOrderBy = JSON.parse(container.dataset.blockProductorderby);
          const productType = JSON.parse(container.dataset.blockProducttype);
          const productStatus = JSON.parse(container.dataset.blockProductstatus);
          const productCategories = JSON.parse(container.dataset.blockProductcategories);
          const productTags = JSON.parse(container.dataset.blockProducttags);
          const productPerPage = JSON.parse(container.dataset.blockProductperpage);
          const customMessage = JSON.parse(container.dataset.blockCustommessage);

          ReactDOM.createRoot(container).render(
              <React.StrictMode>
                  <Table
                    tableColumnsOrder={columnsOrder}
                    tableVisibleColumns={visibileColumns}
                    tableSearch={tableSearch}
                    productCategories={productCategories}
                    productTags={productTags}
                    productOrder={productOrder}
                    productOrderBy={productOrderBy}
                    productType={productType}
                    productStatus={productStatus}
                    productPerPage={productPerPage}
                    customMessage={customMessage}
                  />
              </React.StrictMode>
          );
        }
      });
